import 'dart:async';

import 'package:flutter/material.dart';

import 'interactor.dart';
import 'model/response.model.dart';
import 'pages/payment_page.dart';
import 'utils/api_helper.dart';
import 'utils/end_points.dart';

class PaystackClient {

  static String PUBLIC_KEY = '';
  static String SECRET_KEY = '';

  static void init({secret, public}) async {
    PUBLIC_KEY = public;
    SECRET_KEY = secret;
  }


  static Future _initializePayment(paymentData) async {
    Completer completer = Completer();
    
    try {
      ApiResponse response = await ApiHelper.makePostRequest(url: InitializePaystackTransaction, data: paymentData, headers: {
        'Authorization': 'Bearer $SECRET_KEY'
      });
      if (response.status == true) {
        var res = response.toJson();
        completer.complete((res));
      } else {
        var res = response.toJson();
        completer.complete((res));
      }
    } catch (error) {
      var res = {
        "status": false, "message": "$error", "data": null 
      };
      completer.completeError(Map.from(res));
    }
    return completer.future;
  }


  static Future payWithPayStack({context, paymentData}) async {
    showLoadingDialog(context, "Initializing Payment...");
    var payment = await _initializePayment(paymentData);
    
    Navigator.pop(context);
    if (payment['status'] == true) {
      var paymentResponse = await Navigator.of(context).push(MaterialPageRoute(
        builder: (context) {
          return PaymentPage(paymentData: payment['data'],);
        },
      ));
      if (paymentResponse == null) {
        return {'status': false, 'message': 'The payment was canceled', 'data': paymentResponse};
      } else {
        return await _verifyTransaction(context, paymentResponse);
      }
    } else {
      return payment;
    }
  }

  static Future _verifyTransaction(BuildContext context, paymentData) async {
    try {
      showLoadingDialog(context, "Verifying Transaction...");
      ApiResponse response = await ApiHelper.makeGetRequest(
        url: '$VerifyPaystackTransaction${paymentData['reference']}', headers: {
          'Authorization': 'Bearer $SECRET_KEY'
        }
      );
      Navigator.pop(context);
      // print('Verified Payment: ${response.toJson()}');
      if (response.status) {
        String status = response.data['status'];
        if (status == 'success') {
          showAlertMessage(context, response.data['gateway_response']);
        } else {
          showAlertMessage(context, response.data['gateway_response']);
        }
      } else {
        showAlertMessage(context, response.message);
      }
      return response.toJson();
    } catch (error) {
      showAlertMessage(context, "Error: $error");
      return {'status': false, 'message': 'An error occurred, while making payment', 'data': null};
      // rethrow;
    }
  }
}